/*
 * Copyright E. Bigeon (2014)
 *
 * emmanuel@bigeon.fr
 *
 * This software is a computer program whose purpose is to
 * Socket implementation of GCLC.
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
/**
 * gclc-socket:fr.bigeon.gclc.socket.ConsoleRunnableTest.java
 * Created on: Jun 1, 2016
 */
package fr.bigeon.gclc.socket;

import java.io.IOException;

import org.junit.Test;

import fr.bigeon.gclc.ConsoleApplication;
import fr.bigeon.gclc.manager.ConsoleManager;
import fr.bigeon.gclc.manager.SystemConsoleManager;

/** Test class for {@link ConsoleRunnable}
 *
 * @author Emmanuel Bigeon */
@SuppressWarnings({"static-method", "unused", "javadoc"})
public class ConsoleRunnableTest {

    /** <p>
     * TODO
     *
     * @author Emmanuel Bigeon */
    private static final class ConsoleManagerTestImplementation
            implements ConsoleManager {
        int i = 0;
        String[] cmds;

        /** @param cmds */
        public ConsoleManagerTestImplementation(String[] cmds) {
            super();
            this.cmds = cmds;
        }

        @Override
        public void setPrompt(String prompt) {
            // do nothing
        }

        @Override
        public String getPrompt() {
            // Not used in test
            return ""; //$NON-NLS-1$
        }

        @Override
        public String prompt() {
            try {
                Thread.sleep(1000);
            } catch (InterruptedException e) { // NOSONAR
                // do nothing
            }
            i++;
            if (i == cmds.length) {
                i = 0;
            }
            return cmds[i];
        }

        @Override
        public String prompt(String message) {
            return prompt();
        }

        @Override
        public void println(String message) {
            // do nothing
        }

        @Override
        public void println() {
            // do nothing
        }

        @Override
        public void print(String text) {
            // do nothing
        }

        @Override
        public void close() throws IOException {
            // do nothing
        }
    }

    /** Test method for
     * {@link fr.bigeon.gclc.socket.ConsoleRunnable#ConsoleRunnable(fr.bigeon.gclc.ConsoleApplication, java.lang.Object)}
     * . */
    @Test
    public void testConsoleRunnable() {
        Object lock = new Object();
        ConsoleApplication app = new ConsoleTestApplication(
                new SystemConsoleManager());
        ConsoleRunnable runnable = new ConsoleRunnable(app, lock);

    }

    /** Test method for {@link fr.bigeon.gclc.socket.ConsoleRunnable#run()}. */
    @Test
    public void testRunFlow() {
        Object lock = new Object();
        ConsoleApplication app = new ConsoleTestApplication(
                new ConsoleManagerTestImplementation(
                        new String[] {"test", ConsoleTestApplication.EXIT}));
        ConsoleRunnable runnable = new ConsoleRunnable(app, lock);

        Thread th = new Thread(runnable);
        th.start();

        runnable.stop();
    }

    /** Test method for {@link fr.bigeon.gclc.socket.ConsoleRunnable#stop()}. */
    @Test
    public void testStop() {
        Object lock = new Object();
        ConsoleApplication app = new ConsoleTestApplication(
                new ConsoleManagerTestImplementation(
                        new String[] {"test", ConsoleTestApplication.EXIT}));
        ConsoleRunnable runnable = new ConsoleRunnable(app, lock);
        runnable.stop();
        Thread th = new Thread(runnable);
        th.start();
        runnable.stop();
        runnable.stop();
    }

    /** Test method for {@link fr.bigeon.gclc.socket.ConsoleRunnable#stop()}. */
    @Test
    public void testRun() {
        Object lock = new Object();
        ConsoleApplication app = new ConsoleTestApplication(
                new ConsoleManagerTestImplementation(
                        new String[] {"test", ConsoleTestApplication.EXIT}));
        ConsoleRunnable runnable = new ConsoleRunnable(app, lock);
        runnable.run();
    }

}
