/*
 * Copyright E. Bigeon (2014)
 *
 * emmanuel@bigeon.fr
 *
 * This software is a computer program whose purpose is to
 * Socket implementation of GCLC.
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
package fr.bigeon.gclc.socket;

import fr.bigeon.gclc.ConsoleApplication;
import fr.bigeon.gclc.command.Command;
import fr.bigeon.gclc.command.ICommand;
import fr.bigeon.gclc.exception.InvalidCommandName;
import fr.bigeon.gclc.manager.ConsoleManager;
import fr.bigeon.gclc.prompt.CLIPrompterMessages;

/** A test-purpose application
 * 
 * @author Emmanuel Bigeon */
public class ConsoleTestApplication extends ConsoleApplication {

    /** Exit command */
    public static final String EXIT = "exit"; //$NON-NLS-1$

    /** @param manager the manager */
    @SuppressWarnings("nls")
    public ConsoleTestApplication(final ConsoleManager manager) {
        super(manager, "Welcome to the test application. Type help or test.",
                "See you");
        try {
            add(new ExitCommand(EXIT, this));
            addHelpCommand("help");
            add(new Command("test") {

                @Override
                public String tip() {
                    return "A test command";
                }

                @Override
                public void execute(String... args) {
                    manager.println("Test command ran fine");
                }
            });
        } catch (final InvalidCommandName e) {
            e.printStackTrace();
        }
    }

    /** <p>
     * A command to exit a {@link ConsoleApplication}.
     *
     * @author Emmanuel BIGEON */
    class ExitCommand implements ICommand {
        /** The exit command manual message key */
        private static final String EXIT_MAN = "exit.man"; //$NON-NLS-1$
        /** The tip of the exit command */
        private static final String EXIT_TIP = "exit.tip"; //$NON-NLS-1$
        /** The application that will be exited when this command runs */
        private final ConsoleApplication app;
        /** The exit command name */
        private final String name;

        /** @param name the name of the command
         * @param app the application to exit */
        public ExitCommand(String name, ConsoleApplication app) {
            this.name = name;
            this.app = app;
        }

        /** The actions to take before exiting */
        public void beforeExit() {
            // Do nothing by default
        }

        @Override
        public final void execute(String... args) {
            beforeExit();
            app.exit();
        }

        /* (non-Javadoc)
         * @see fr.bigeon.gclc.command.ICommand#getCommandName() */
        @Override
        public String getCommandName() {
            return name;
        }

        @Override
        public void help(ConsoleManager helpManager, String... args) {
            helpManager.println(
                    CLIPrompterMessages.getString(EXIT_MAN, (Object[]) args));
        }

        @Override
        public String tip() {
            return CLIPrompterMessages.getString(EXIT_TIP);
        }
    }
}
